// ########################################################################
// ## anzahltabelle.cc:                                                  ##
// ## Klasse zur Verwaltung der Anzahltabelle fr RandCubic              ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>


#include "anzahltabelle.h"
#include "../system.h"


// Konstruktor
Anzahltabelle::Anzahltabelle()
{
   eintraege = AnzahlTabelleGroesse;

   // Speicher beschaffen und mit 0 initialisieren
   speicher = new AnzahlTyp[(eintraege + 6) * 4];
   for(card32 i = 0;i < (eintraege + 6) * 4;i++) {
      speicher[i] = 0;
   }
   // Einzelne Tabellen anlegen. Fr negative Indizes Platz schaffen.
   tabelleA = &speicher[5];
   tabelleB = &speicher[5 + (eintraege + 6)];
   tabelleC = &speicher[5 + 2 * (eintraege + 6)];
   tabelleD = &speicher[5 + 3 * (eintraege + 6)];
   tabelleD[0] = 1;   // Konvention: Leerer Graph ist kubisch

   // Einzelne Eintrge rekursiv berechnen
   for(card32 n = 4;n <= eintraege;n++) {
      // Fr gerade Eintrge: a[n] rekursiv berechnen
      if((n % 2) != 0) {
         tabelleA[n] = n * tabelleB[n - 1] / 2;
      }
      // Fr ungerade Eintrge: b[n], c[n] und d[n] rekursiv berechnen
      else {
         tabelleC[n] =
            (AnzahlTyp)(((3 * n - 8) / 2) * n)                           * tabelleA[n - 1] +
            (AnzahlTyp)(n * (n - 1))                                     * tabelleB[n - 2] +
            (AnzahlTyp)((n * (n - 1) * (n - 2) * (n - 3)) / 2)           * tabelleD[n - 4] +
            (AnzahlTyp)((n * (n - 1) * (n - 2) * (n - 3) * (n - 4)) / 2) * tabelleA[n - 5] +
            (AnzahlTyp)((n * (n - 1) * (n - 2) * (n - 3)) / 2)           * tabelleB[n - 4];
         tabelleB[n] =
            tabelleC[n]                                          +
            (AnzahlTyp)(n * (n - 1) * (n - 2)) * tabelleA[n - 3] +
            (AnzahlTyp)(n * (n - 1))           * tabelleB[n - 2];
         tabelleD[n] =
            tabelleC[n] / (AnzahlTyp)(3 * n);
      }
   }
}


// Destruktor
Anzahltabelle::~Anzahltabelle()
{
   delete speicher;
   speicher = tabelleA = tabelleB = tabelleC = tabelleD = NULL;
}


// Ausgabe-Operator
ostream& operator<<(ostream& os, const Anzahltabelle& tabelle)
{
   for(card32 i = 0;i <= tabelle.eintraege;i++) {
      os << i << ": ";
      os << tabelle.tabelleA[i] << "  ";
      os << tabelle.tabelleB[i] << "  ";
      os << tabelle.tabelleC[i] << "  ";
      os << tabelle.tabelleD[i] << endl;
   }
   return os;
}
